<?php

namespace Teknisa\CADEST\Factories;


use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Ger03800ProdutosPorUnidadeFactory {
    protected $connection;

    public function __construct($entityManager) {
        $this->connection = $entityManager->getConnection();
        date_default_timezone_set('America/Sao_Paulo');
    }

    public function getProdutos($params) {
        $sql = "SELECT P.NRORG,
                 P.CDFILIAL,
                 P.CDPRODUTO,
                 P.IDCNTRESTOQ,
                 PR.NMPRODUTO,
                 P.NRDIASESTOQ,
                 PR.CDARVPROD,
                 P.IDUTCTRLLOTE,
                 P.IDUTVRFTCONVVAR,
                 P.IDPRODCOMRASTRO,
                 P.NRDIASLIMSEGVAL
        FROM PRODFILI P, PRODUTO PR WHERE(P.CDPRODUTO = PR.CDPRODUTO)
                AND (P.CDFILIAL = :CDFILIAL)
                AND (NVL(PR.IDPRODATIVPR,'S') = 'S')
                AND (PR.CDPRODUTO = PR.CDPRODESTO)
                AND ((PR.CDARVPROD
            BETWEEN :P_CDPRODINI
                AND RPAD(:P_CDPRODFIN, 20, 'z'))
                OR (:TODOS = 'T'))
        ORDER BY  PR.CDARVPROD";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getProdFili($params) {
        $sql = "SELECT P.CDFILIAL,
                  P.CDPRODUTO,
                  P.IDPRODATIV,
                  P.IDUTILPESAGEM,
                  P.VRPTOLQTITAF,
                  P.VRPTOLVRITAF,
                  P.VRTXADMPROD,
                  PR.NMPRODUTO,
                  PR.CDARVPROD,
                  P.VRPTOLAJUSINV,
                  P.IDUTPRORIGOBRIG,
                  P.VRMINEMPENHOAF,
                  P.VRMINEMPENHOOL,
                  P.NRORG
                FROM PRODFILI P,
                  PRODUTO PR
                WHERE(P.CDFILIAL     = :CDFILIAL)
                AND (P.CDPRODUTO     = PR.CDPRODUTO)
                AND (PR.IDPRODATIVPR = 'S')
                AND ((PR.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN||'zzzzzzzzzzzzzzzzzzzz')
                OR (:TODOS = 'T'))
                ORDER BY PR.CDARVPROD";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function deletaPosEstDiaFil($cdFilial, $cdProdIni, $cdProdFin) {
        $sql = "DELETE
                FROM POSESTDIA
                WHERE (CDFILIAL = :P_CDFILIAL)
                AND CDPRODUTO  IN
                  (SELECT CDPRODUTO
                  FROM PRODFILI P
                  WHERE (P.CDFILIAL  = :P_CDFILIAL)
                  AND (P.IDCNTRESTOQ = 'N')
                  AND EXISTS
                    (SELECT CDPRODUTO
                    FROM PRODUTO
                    WHERE (CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN)
                    AND (PRODUTO.CDPRODUTO    = P.CDPRODUTO)
                    AND (PRODUTO.IDPRODATIVPR = 'S')
                    AND (CDNVPRODUTO          =
                      (SELECT MAX(CDNVPRODUTO) FROM PRODUTO
                      ))
                    )
                  )";
                  
        $params = array( 'P_CDFILIAL'    => $cdFilial,
                         'P_PRODINI'    => $cdProdIni,
                         'P_PRODFIN'    => $cdProdFin);
                         
        $std = $this->connection->prepare($sql);
        return $std->execute($params);
    }
    
    public function deletaPosEstConsFil($cdFilial, $cdProdIni, $cdProdFin) {
        $sql = "DELETE
                FROM POSESTCONS
                WHERE (CDFILIAL = :P_CDFILIAL)
                AND CDPRODUTO  IN
                  (SELECT CDPRODUTO
                  FROM PRODFILI P
                  WHERE (P.CDFILIAL  = :P_CDFILIAL)
                  AND (P.IDCNTRESTOQ = 'N')
                  AND EXISTS
                    (SELECT CDPRODUTO
                    FROM PRODUTO
                    WHERE (CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN)
                    AND (PRODUTO.CDPRODUTO    = P.CDPRODUTO)
                    AND (PRODUTO.IDPRODATIVPR = 'S')
                    AND (CDNVPRODUTO          =
                      (SELECT MAX(CDNVPRODUTO) FROM PRODUTO
                      ))
                    )
                  )";
                  
        $params = array( 'P_CDFILIAL'    => $cdFilial,
                         'P_PRODINI'    => $cdProdIni,
                         'P_PRODFIN'    => $cdProdFin);
                         
        $std = $this->connection->prepare($sql);
        return $std->execute($params);
    }
    
    public function deletaPosEstAtuFil($cdFilial, $cdProdIni, $cdProdFin) {
        $sql = "DELETE
                FROM POSESTATU
                WHERE (CDFILIAL = :P_CDFILIAL)
                AND CDPRODUTO  IN
                  (SELECT CDPRODUTO
                  FROM PRODFILI P
                  WHERE (P.CDFILIAL  = :P_CDFILIAL)
                  AND (P.IDCNTRESTOQ = 'N')
                  AND EXISTS
                    (SELECT CDPRODUTO
                    FROM PRODUTO
                    WHERE (CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN)
                    AND (PRODUTO.CDPRODUTO    = P.CDPRODUTO)
                    AND (PRODUTO.IDPRODATIVPR = 'S')
                    AND (CDNVPRODUTO          =
                      (SELECT MAX(CDNVPRODUTO) FROM PRODUTO
                      ))
                    )
                  )";
        
        $params = array( 'P_CDFILIAL'    => $cdFilial,
                         'P_PRODINI'    => $cdProdIni,
                         'P_PRODFIN'    => $cdProdFin);
                         
        $std = $this->connection->prepare($sql);
        return $std->execute($params);
    }
    
    public function deletaPosEstDiaGrupFil($cdFilial, $cdProdIni, $cdProdFin) {
        $sql = "DELETE
                FROM POSESTDIA
                WHERE CDFILIAL IN
                  (SELECT CDFILIAL FROM FILGRUPFILI WHERE CDGRUPFILI = :P_CDFILIAL
                  )
                AND CDPRODUTO IN
                  (SELECT CDPRODUTO
                  FROM PRODFILI P
                  WHERE CDFILIAL IN
                    (SELECT CDFILIAL FROM FILGRUPFILI WHERE CDGRUPFILI = :P_CDFILIAL
                    )
                  AND (P.IDCNTRESTOQ = 'N')
                  AND EXISTS
                    (SELECT CDPRODUTO
                    FROM PRODUTO
                    WHERE (CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN)
                    AND (PRODUTO.CDPRODUTO    = P.CDPRODUTO)
                    AND (PRODUTO.IDPRODATIVPR = 'S')
                    AND (CDNVPRODUTO          =
                      (SELECT MAX(CDNVPRODUTO) FROM PRODUTO
                      ))
                    )
                  )";
        
        $params = array( 'P_CDFILIAL'    => $cdFilial,
                         'P_PRODINI'    => $cdProdIni,
                         'P_PRODFIN'    => $cdProdFin);
                         
        $std = $this->connection->prepare($sql);
        return $std->execute($params);
    }
    
    public function deletaPosEstConsGrupFil($cdFilial, $cdProdIni, $cdProdFin) {
        $sql = "DELETE
                FROM POSESTCONS
                WHERE CDFILIAL IN
                  (SELECT CDFILIAL FROM FILGRUPFILI WHERE CDGRUPFILI = :P_CDFILIAL
                  )
                AND CDPRODUTO IN
                  (SELECT CDPRODUTO
                  FROM PRODFILI P
                  WHERE CDFILIAL IN
                    (SELECT CDFILIAL FROM FILGRUPFILI WHERE CDGRUPFILI = :P_CDFILIAL
                    )
                  AND (P.IDCNTRESTOQ = 'N')
                  AND EXISTS
                    (SELECT CDPRODUTO
                    FROM PRODUTO
                    WHERE (CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN)
                    AND (PRODUTO.CDPRODUTO    = P.CDPRODUTO)
                    AND (PRODUTO.IDPRODATIVPR = 'S')
                    AND (CDNVPRODUTO          =
                      (SELECT MAX(CDNVPRODUTO) FROM PRODUTO
                      ))
                    )
                  )";
        $params = array( 'P_CDFILIAL'    => $cdFilial,
                         'P_PRODINI'    => $cdProdIni,
                         'P_PRODFIN'    => $cdProdFin);
                         
        $std = $this->connection->prepare($sql);
        return $std->execute($params);           
    }
    
    public function deletaPosEstAtuGrupFil($cdFilial, $cdProdIni, $cdProdFin) {
        $sql = "DELETE
                FROM POSESTATU
                WHERE CDFILIAL IN
                  (SELECT CDFILIAL FROM FILGRUPFILI WHERE CDGRUPFILI = :P_CDFILIAL
                  )
                AND CDPRODUTO IN
                  (SELECT CDPRODUTO
                  FROM PRODFILI P
                  WHERE CDFILIAL IN
                    (SELECT CDFILIAL FROM FILGRUPFILI WHERE CDGRUPFILI = :P_CDFILIAL
                    )
                  AND (P.IDCNTRESTOQ = 'N')
                  AND EXISTS
                    (SELECT CDPRODUTO
                    FROM PRODUTO
                    WHERE (CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN)
                    AND (PRODUTO.CDPRODUTO    = P.CDPRODUTO)
                    AND (PRODUTO.IDPRODATIVPR = 'S')
                    AND (CDNVPRODUTO          =
                      (SELECT MAX(CDNVPRODUTO) FROM PRODUTO
                      ))
                    )
                  )";
        
        $params = array( 'P_CDFILIAL'    => $cdFilial,
                         'P_PRODINI'    => $cdProdIni,
                         'P_PRODFIN'    => $cdProdFin);
                         
        $std = $this->connection->prepare($sql);
        return $std->execute($params);           
    }
    
    public function deletaPosEstDiaLstFil($cdFilial, $cdProdIni, $cdProdFin) {
        $sql = "DELETE
                FROM POSESTDIA
                WHERE (INSTR(:P_CDFILIAL, '_'
                  ||CDFILIAL
                  ||'_')       > 0)
                AND CDPRODUTO IN
                  (SELECT CDPRODUTO
                  FROM PRODFILI P
                  WHERE (INSTR(:P_CDFILIAL, '_'
                    ||CDFILIAL
                    ||'_')           > 0)
                  AND (P.IDCNTRESTOQ = 'N')
                  AND EXISTS
                    (SELECT CDPRODUTO
                    FROM PRODUTO
                    WHERE (CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN)
                    AND (PRODUTO.CDPRODUTO    = P.CDPRODUTO)
                    AND (PRODUTO.IDPRODATIVPR = 'S')
                    AND (CDNVPRODUTO          =
                      (SELECT MAX(CDNVPRODUTO) FROM PRODUTO
                      ))
                    )
                  )";
                  
        $params = array( 'P_CDFILIAL'    => $cdFilial,
                         'P_PRODINI'    => $cdProdIni,
                         'P_PRODFIN'    => $cdProdFin);
                         
        $std = $this->connection->prepare($sql);
        return $std->execute($params);     
    }
    
    public function deletaPosEstConsLstFil($cdFilial, $cdProdIni, $cdProdFin) {
        $sql = "DELETE
                FROM POSESTCONS
                WHERE (INSTR(:P_CDFILIAL, '_'
                  ||CDFILIAL
                  ||'_')       > 0)
                AND CDPRODUTO IN
                  (SELECT CDPRODUTO
                  FROM PRODFILI P
                  WHERE (INSTR(:P_CDFILIAL, '_'
                    ||CDFILIAL
                    ||'_')           > 0)
                  AND (P.IDCNTRESTOQ = 'N')
                  AND EXISTS
                    (SELECT CDPRODUTO
                    FROM PRODUTO
                    WHERE (CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN)
                    AND (PRODUTO.CDPRODUTO    = P.CDPRODUTO)
                    AND (PRODUTO.IDPRODATIVPR = 'S')
                    AND (CDNVPRODUTO          =
                      (SELECT MAX(CDNVPRODUTO) FROM PRODUTO
                      ))
                    )
                  )";
                  
        $params = array( 'P_CDFILIAL'    => $cdFilial,
                         'P_PRODINI'    => $cdProdIni,
                         'P_PRODFIN'    => $cdProdFin);
                         
        $std = $this->connection->prepare($sql);
        return $std->execute($params);  
    }
    
    public function deletaPosEstAtuLstFil($cdFilial, $cdProdIni, $cdProdFin) {
         $sql = "DELETE
                FROM POSESTATU
                WHERE (INSTR(:P_CDFILIAL, '_'
                  ||CDFILIAL
                  ||'_')       > 0)
                AND CDPRODUTO IN
                  (SELECT CDPRODUTO
                  FROM PRODFILI P
                  WHERE (INSTR(:P_CDFILIAL, '_'
                    ||CDFILIAL
                    ||'_')           > 0)
                  AND (P.IDCNTRESTOQ = 'N')
                  AND EXISTS
                    (SELECT CDPRODUTO
                    FROM PRODUTO
                    WHERE (CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN)
                    AND (PRODUTO.CDPRODUTO    = P.CDPRODUTO)
                    AND (PRODUTO.IDPRODATIVPR = 'S')
                    AND (CDNVPRODUTO          =
                      (SELECT MAX(CDNVPRODUTO) FROM PRODUTO
                      ))
                    )
                  )";
                  
        $params = array( 'P_CDFILIAL'    => $cdFilial,
                         'P_PRODINI'    => $cdProdIni,
                         'P_PRODFIN'    => $cdProdFin);
                         
        $std = $this->connection->prepare($sql);
        return $std->execute($params);  
    }
   
   
    public function getProdControlEstoque($cdFilial, $cdProdIni, $cdProdFin) { 
        $sql = "SELECT PF.CDPRODUTO
                    FROM PRODFILI PF, PRODUTO P
                   WHERE P.CDPRODUTO = PF.CDPRODUTO
                     AND NVL(PF.IDCNTRESTOQ, 'S') = 'S'
                     AND PF.CDFILIAL = :P_CDFILIAL
                     AND P.CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN";
        
        $params = array( 'P_CDFILIAL'    => $cdFilial,
                         'P_PRODINI'    => $cdProdIni,
                         'P_PRODFIN'    => $cdProdFin);
                         
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getFiliaisGrupFili($cdGrupFili) {
        $sql = "SELECT CDFILIAL
                FROM FILGRUPFILI
               WHERE CDGRUPFILI = :P_CDGRUPFILI";
        
        $params = array('P_CDGRUPFILI'    => $cdGrupFili);
                         
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function insereMoviLote($cdFilial, $cdProdIni, $cdProdFin, $nrMoviLote, $nrOrg) {
        $sql = "INSERT
                INTO MOVILOTE
                  (
                    NRMOVILOTE,
                    CDFILIAL,
                    CDALMOXARIFE,
                    CDLOCALESTOQ,
                    NRLOTEESTQ,
                    NRSUBLOTE,
                    CDPRODUTO,
                    NRLANCTOORIG,
                    NRORG
                  )
                  (SELECT DISTINCT :P_NRMOVILOTE AS NRMOVILOTE,
                      IL.CDFILIMOVI,
                      IL.CDALMOXARIFE,
                      IL.CDLOCALESTOQ,
                      IL.NRLOTEESTQ,
                      IL.NRSUBLOTE,
                      IL.CDPRODMOVI,
                      :P_NRMOVILOTE,
                      :P_NRORG
                    FROM ITLANCTOEST IL,
                      PRODFILI PF,
                      PRODUTO PR
                    WHERE (IL.CDFILIMOVI = :P_CDFILIAL )
                    AND (PF.CDFILIAL     = :P_CDFILIAL )
                    AND (IL.CDPRODMOVI   = PF.CDPRODUTO )
                    AND (PF.CDPRODUTO    = PR.CDPRODUTO )
                    AND (PR.CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIM )
                    AND (NVL(IL.IDEVOLESTOQ,'S')  = 'S' )
                    AND (NVL(PF.IDCNTRESTOQ, 'S') = 'S' )
                    GROUP BY IL.CDFILIMOVI,
                      IL.CDALMOXARIFE,
                      IL.CDLOCALESTOQ,
                      IL.NRLOTEESTQ,
                      IL.NRSUBLOTE,
                      IL.CDPRODMOVI,
                      PR.CDARVPROD,
                      PR.NMPRODUTO
                  )";
                  
        $params = array( 'P_CDFILIAL'   => $cdFilial,
                         'P_PRODINI'    => $cdProdIni,
                         'P_PRODFIM'    => $cdProdFin,
                         'P_NRMOVILOTE' => $nrMoviLote,
                         'P_NRORG'      => $nrOrg);
                         
        $std = $this->connection->prepare($sql);
        return $std->execute($params);
 
    }
    
    public function deletaMoviLote($nrMoviLote) {
        $sql = "DELETE MOVILOTE WHERE NRMOVILOTE = :P_NRMOVILOTE"; 
        
        $params = array( 'P_NRMOVILOTE' => $nrMoviLote);
                         
        $std = $this->connection->prepare($sql);
        return $std->execute($params);
    }
    
    public function callProcedureAtualizaProdFili($cdFiliOrig, $cdFiliDest, $idTpFiliDest, $idTpConfPar, $cdProdIni, $cdProdFin, $listaParam) {
        $procedure = new StoredProcedure($this->connection, 'REPLICA_PRODUTO_FILIAL');
        
        $cdProdfin = str_pad($cdProdFin, 20, "0", STR_PAD_LEFT); 

        $procedure->addParam(new Param('P_CDFILIORIG',  Param::PARAM_INPUT,     $cdFiliOrig,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIDEST',  Param::PARAM_INPUT,     $cdFiliDest,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTPFILDEST', Param::PARAM_INPUT,     $idTpFiliDest,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTPCONFPAR', Param::PARAM_INPUT,     $idTpConfPar,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODINIC',  Param::PARAM_INPUT,     $cdProdIni,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODFINA',  Param::PARAM_INPUT,     $cdProdFin,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LISTAPARAM',  Param::PARAM_INPUT,     $listaParam,    Param::PARAM_TYPE_STR));

        $procedure->execute();
    }
    
    public function callProcedureAtualizaEstoque($cdFilial, $dtFinal, $nrLancto) {
        $procedure = new StoredProcedure($this->connection, 'ATUALIZA_ESTOQUE');

        $procedure->addParam(new Param('P_FILIMOVI',    Param::PARAM_INPUT,     $cdFilial,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL',   Param::PARAM_INPUT,     '30/12/1899',   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINAL',     Param::PARAM_INPUT,     $dtFinal,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILILANCTO',  Param::PARAM_INPUT,     $cdFilial,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTO',    Param::PARAM_INPUT,     $nrLancto,      Param::PARAM_TYPE_STR));

        $procedure->execute();
    }
    
    public function validaProdutosComSaldo($cdfilial, $cdprodini, $cdprodfin){
        $sql = "SELECT B.CDPRODUTO, A.CDPRODESTO, A.NMPRODUTO, MAX(B.QTESTOQDIA) AS QTESTOQDIA
                  FROM POSESTDIA B,
                       (SELECT CDPRODUTO, NMPRODUTO, SGUNIDADE, CDARVPROD, CDPRODESTO
                          FROM PRODUTO
                         WHERE CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN) A,
                       (SELECT CDFILIAL, CDPRODUTO, NRLOTEESTQ, NRSUBLOTE, MAX(DTPOSIESTQ) AS DTPOSIESTQ
                          FROM POSESTDIA
                         GROUP BY CDFILIAL, CDPRODUTO, NRLOTEESTQ, NRSUBLOTE) D
                 WHERE B.CDFILIAL = :P_FILIAL
                   AND ABS(B.QTESTOQDIA) > 0.00001
                   AND B.CDPRODUTO = A.CDPRODUTO
                   AND A.CDPRODUTO = A.CDPRODESTO
                   AND B.CDFILIAL = D.CDFILIAL
                   AND B.CDPRODUTO = D.CDPRODUTO
                   AND B.NRLOTEESTQ = D.NRLOTEESTQ
                   AND B.NRSUBLOTE = D.NRSUBLOTE
                   AND B.DTPOSIESTQ = D.DTPOSIESTQ
                 GROUP BY B.CDPRODUTO, A.NMPRODUTO, A.SGUNIDADE, A.CDPRODESTO, A.CDARVPROD
                 ORDER BY A.NMPRODUTO";
        
        $params = array('P_PRODINI'    => $cdprodini,
                        'P_PRODFIN'    => $cdprodfin,
                        'P_FILIAL'    => $cdfilial
                  );
                         
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $resultado = $std->fetchAll();
        if(count($resultado))
          throw new \Exception ('Operação Bloqueada! Alteração requer que os produtos indicados sejam zerados no estoque. É recomendado que antes de zera-los seja emitida a posição atual dos mesmos na filial para posterior implantação.');
    }
    
    public function validaAlteraLote($cdfilial, $cdprodini, $cdprodfin){
        $sql = "SELECT P.CDFILIAL, PR.CDARVPROD, P.IDCNTRESTOQ
                  FROM PRODFILI P, PRODUTO PR
                 WHERE (P.CDPRODUTO = PR.CDPRODUTO)
                   AND (P.CDFILIAL = :P_FILIAL)
                   AND (NVL(PR.IDPRODATIVPR,'S') = 'S')
                   AND (PR.CDPRODUTO = PR.CDPRODESTO)
                   AND (PR.CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN)
                   AND (P.IDCNTRESTOQ <> 'S')
                 ORDER BY PR.CDARVPROD";
        
        $params = array('P_PRODINI'    => $cdprodini,
                        'P_PRODFIN'    => $cdprodfin,
                        'P_FILIAL'    => $cdfilial
                  );
                         
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $resultado = $std->fetchAll();
        if(count($resultado))
          throw new \Exception ('Operação bloqueada! Alteração requer que os produtos indicados sejam controlados pelo estoque.');
    }
}